@@ -1,8 +1,6 @@
-from explorer.utils import passes_blacklist
-from django.db import connection, DatabaseError, models
+from explorer.utils import passes_blacklist, write_csv, swap_params, execute_query, execute_and_fetch_query, extract_params, shared_dict_update
+from django.db import models, DatabaseError
 from django.core.urlresolvers import reverse
-import csv
-import cStringIO
 
 MSG_FAILED_BLACKLIST = "Query failed the SQL blacklist."
 
@@ -14,6 +12,8 @@ class Query(models.Model):
     created_by = models.CharField(max_length=255, null=True, blank=True)
     created_at = models.DateTimeField(auto_now_add=True)
 
+    params = {}
+
     class Meta:
         ordering = ['title']
     
@@ -21,28 +21,39 @@ class Query(models.Model):
         return unicode(self.title)
 
     def passes_blacklist(self):
-        return passes_blacklist(self.sql)
+        return passes_blacklist(self.final_sql())
+
+    def final_sql(self):
+        return swap_params(self.sql, self.params)
 
     def csv_report(self):
-        csv_report = cStringIO.StringIO()
-        writer = csv.writer(csv_report)
         headers, data, error = self.headers_and_data()
-        if error: return error
-        writer.writerow(headers)
-        map(lambda row: writer.writerow(row), data)
-        return csv_report.getvalue()
+        if error:
+            return error
+        return write_csv(headers, data)
+
+    def error_messages(self):
+        if not self.passes_blacklist():
+            return MSG_FAILED_BLACKLIST
+        try:
+            execute_query(self.final_sql())
+            return None
+        except DatabaseError, e:
+            return str(e)
 
     def headers_and_data(self):
         if not self.passes_blacklist():
             return [], [], MSG_FAILED_BLACKLIST
-        cursor = connection.cursor()
         try:
-            cursor.execute(self.sql)
+            return execute_and_fetch_query(self.final_sql())
         except DatabaseError, e:
-            return [], [], e
-        headers = [d[0] for d in cursor.description]
-        data = [[x.encode('utf-8') if type(x) is unicode else x for x in list(r)] for r in cursor.fetchall()]
-        return headers, data, None
+            return [], [], str(e)
+
+    def available_params(self):
+        p = extract_params(self.sql)
+        if self.params:
+            shared_dict_update(p, self.params)
+        return p
 
     def get_absolute_url(self):
         return reverse("query_detail", kwargs={'query_id': self.id})
\ No newline at end of file
